        include "mc68332.inc"


MAX_TPU_CHAN    equ     15


; The following register pairs will be accessed using longword accesses:
;
;     TPU_CFSR0/TPU_CFSR1
;     TPU_CFSR2/TPU_CFSR3
;     TPU_HSQR0/TPU_HSQR1
;     TPU_HSRR0/TPU_HSRR1
;     TPU_CPR0/TPU_CPR1
;
; The TMCR, TICR, and CIER registers will be accessed using word accesses. The
; CISR register will be accessed using byte accesses.

        section code

; Function: void tpu_init( int TMCR_value, int vector, int level )
;
; This function initializes the TPU for operation. It takes a two-byte integer
; value from which will be placed into the TPU Module Configuration Register
; (TMCR). It takes two other integers: the second specifies the base interrupt
; vector for the TPU, and the third specifies the level of the TPU interrupts.

        XDEF    _tpu_init

_tpu_init:
        move.l  #$FFFF00,a0
        move.l  #(8*16)-1,d0
tpu_init_1:
        move.w  #0,(a0)+
        dbra    d0,tpu_init_1

        move.l  #$FFF000,a0
        move.w  4(a7),TPU_TMCR(a0)
        move.w  6(a7),d0
        move.w  8(a7),d1
        andi.l  #$0F,d1
        andi.l  #$F0,d0
        lsl.w   #8,d1
        or.w    d1,d0
        move.w  d0,TPU_TICR(a0)
        rts


; Function: void tpu_cfs_set( int channel, int function )
;
; This function sets a four-bit "channel function selection" for the specified
; channel. It takes two arguments, which are presumed to be two-byte integers.

        XDEF    _tpu_cfs_set

_tpu_cfs_set:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit channel number
        move.l  #$FFF000,a0
        bclr.l  #3,d1                   ; test if channels 8-15
        bne.s   tcs.0
        addq.l  #4,a0                   ; channels 0-7
tcs.0   lsl.l   #2,d1                   ; 4 bits/channel
        moveq.l #15,d0                  ; load the mask
        lsl.l   d1,d0                   ; shift into position
        not.l   d0                      ; invert the mask
        and.l   d0,TPU_CFSR0(a0)        ; clear the previous function
        moveq.l #15,d0                  ; load up the mask again
        and.w   6(a7),d0                ; load new channel function value
        lsl.l   d1,d0                   ; move into position
        or.l    d0,TPU_CFSR0(a0)        ; put into register
        rts


; Function: int tpu_cfs_get( int channel )
;
; This function returns the four-bit "channel function select" code for the
; specified channel. The input argument is presumed to be a two-byte integer.

        XDEF    _tpu_cfs_get

_tpu_cfs_get:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit channel number
        move.l  #$FFF000,a0
        bclr.l  #3,d1                   ; test if channels 8-15
        bne.s   tcg.0
        addq.l  #4,a0                   ; channels 0-7
tcg.0   lsl.l   #2,d1                   ; 4 bits/channel
        move.l  TPU_CFSR0(a0),d0        ; get the info
        lsr.l   d1,d0                   ; shift down into LSBs
        andi.l  #15,d0                  ; mask off everything else
        rts


; Function: void tpu_hsq_set( int channel, int value )
;
; This function sets a two-bit "host sequence request" code for the specified
; channel. It takes two arguments, which are presumed to be two-byte integers.

        XDEF    _tpu_hsq_set

_tpu_hsq_set:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit channel number
        lsl.l   #1,d1                   ; 2 bits/channel
        moveq.l #3,d0                   ; get mask
        lsl.l   d1,d0                   ; shift into position
        not.l   d0                      ; invert mask
        and.l   d0,TPU_HSQR0+$FFF000    ; clear previous request
        move.w  6(a7),d0                ; get new host sequence request
        andi.l  #3,d0                   ; limit to legal value
        lsl.l   d1,d0                   ; move into position
        or.l    d0,TPU_HSQR0+$FFF000    ; put into register
        rts


; Function: int tpu_hsq_get( int channel )
;
; This function returns the two-bit "host sequence request" code for the
; specified channel. The input argument is presumed to be a two-byte integer.

        XDEF    _tpu_hsq_get

_tpu_hsq_get:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit it
        lsl.l   #1,d1                   ; 2 bits/channel
        move.l  TPU_HSQR0+$FFF000,d0    ; get HSQ register values
        lsr.l   d1,d0                   ; shift down to LSBs
        andi.l  #3,d0                   ; mask off everything else
        rts


; Function: void tpu_hsr_set( int channel, int value )
;
; This function sets a two-bit "host service request" code for the specified
; channel. It takes two arguments, which are presumed to be two-byte integers.

        XDEF    _tpu_hsr_set

_tpu_hsr_set:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit channel number
        lsl.l   #1,d1                   ; 2 bits/channel
        moveq.l #3,d0                   ; get mask
        lsl.l   d1,d0                   ; shift into position
        not.l   d0                      ; invert mask
        and.l   d0,TPU_HSRR0+$FFF000    ; clear previous reqquest
        move.w  6(a7),d0                ; get new host sequence request
        andi.l  #3,d0                   ; limit to legal value
        lsl.l   d1,d0                   ; move into position
        or.l    d0,TPU_HSRR0+$FFF000    ; put into register
        rts


; Function: int tpu_hsr_get( int channel )
;
; This function returns the two-bit "host service request" code for the
; specified channel. The input argument is presumed to be a two-byte integer.

        XDEF    _tpu_hsr_get

_tpu_hsr_get:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit it
        lsl.l   #1,d1                   ; 2 bits/channel
        move.l  TPU_HSRR0+$FFF000,d0    ; get HSR register values
        lsr.l   d1,d0                   ; shift down to LSBs
        andi.l  #3,d0                   ; mask off everything else
        rts


; Function: void tpu_cpr_set( int channel, int value )
;
; This function sets a two-bit channel priority code for the specified
; channel. It takes two arguments, which are presumed to be two-byte integers.

        XDEF    _tpu_cpr_set

_tpu_cpr_set:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit channel number
        lsl.l   #1,d1                   ; 2 bits/channel
        moveq.l #3,d0                   ; get mask
        lsl.l   d1,d0                   ; shift into position
        not.l   d0                      ; invert mask
        and.l   d0,TPU_CPR0+$FFF000     ; clear previous request
        move.w  6(a7),d0                ; get new host sequence request
        andi.l  #3,d0                   ; limit to legal value
        lsl.l   d1,d0                   ; move into position
        or.l    d0,TPU_CPR0+$FFF000     ; put into register
        rts


; Function: int tpu_cpr_get( int channel )
;
; This function returns the two-bit channel priority code for the specified
; channel. The input argument is presumed to be a two-byte integer.

        XDEF    _tpu_cpr_get

_tpu_cpr_get:
        move.w  4(a7),d1                ; get channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit it
        lsl.l   #1,d1                   ; 2 bits/channel
        move.l  TPU_CPR0+$FFF000,d0     ; get CPR register values
        lsr.l   d1,d0                   ; shift down to LSBs
        andi.l  #3,d0                   ; mask off everything else
        rts


; Function: void tpu_int_enable( int channel )
;
; This function sets to "1" a one-bit interrupt enable flag for the specified
; channel. It takes one argument, which is presumed to be a two-byte integer.

        XDEF    _tpu_int_enable

_tpu_int_enable:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit to a legal value
        moveq.l #1,d0                   ; get the mask
        lsl.w   d1,d0                   ; shift into position
        or.w    d0,TPU_CIER+$FFF000     ; set the enable bit
        rts


; Function: void tpu_int_disable( int channel )
;
; This function sets to "0" a one-bit interrupt enable flag for the specified
; channel. It takes one argument, which is presumed to be a two-byte integer.

        XDEF    _tpu_int_disable

_tpu_int_disable:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit to a legal value
        moveq.l #1,d0                   ; get the mask
        lsl.w   d1,d0                   ; shift into position
        not.w   d0                      ; invert
        and.w   d0,TPU_CIER+$FFF000     ; clear the enable bit
        rts


; Function: int tpu_int_setting( int channel )
;
; This functions returns the one-bit interrupt enable flag for the specified
; channel. It takes one argument, which is presumed to be a two-byte integer.

        XDEF    _tpu_int_setting

_tpu_int_setting:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit to a legal value
        move.w  TPU_CIER+$FFF000,d0     ; get the current settings
        btst.l  d1,d0                   ; check the channel setting
        sne     d0                      ; set a flag
        neg.b   d0                      ; convert to "proper" flag
        extb.l  d0
        rts


; Function: int tpu_int_status( int channel )
;
; This functions returns the one-bit interrupt status flag for the specified
; channel. It takes one argument, which is presumed to be a two-byte integer.

        XDEF    _tpu_int_status

_tpu_int_status:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit to a legal value
        move.l  #$FFF000,a0
        bclr.l  #3,d1                   ; check for channels 8-15
        bne.s   tis.0
        addq.l  #1,a0                   ; channels 0-7
tis.0   btst.b  d1,TPU_CISR(a0)
        sne     d0                      ; prepare the return value
        neg.b   d0
        extb.l  d0
        rts


; Function: void tpu_int_clear( int channel )
;
; This function sets to "0" the one-bit interrupt status flag for the specified
; channel. It takes one argument, which is presumed to be a two-byte integer.

        XDEF    _tpu_int_clear

_tpu_int_clear:
        move.w  4(a7),d1
        andi.l  #MAX_TPU_CHAN,d1
        moveq.l #1,d0                   ; get the mask
        lsl.w   d1,d0                   ; shift into position
        not.w   d0                      ; invert
        and.w   d0,TPU_CISR+$FFF000     ; clear the enable bit
        rts


; Function: int tpu_int_setup( int channel, void *function )
;
; This function sets up an interrupt handler for a TPU channel. If the vector
; could be set up without problems, a zero will be returned. If not, a -1 will
; be returned. The reason for failure will be a TPU base interrupt vector which
; is set to a value less than 0x40. This function takes two arguments; a two-
; byte integer channel number, and a four-byte function address.

        XDEF    _tpu_int_setup

_tpu_int_setup:
        move.w  4(a7),d1                ; get the channel number
        andi.l  #MAX_TPU_CHAN,d1        ; limit to a legal value
        lsl.l   #2,d1                   ; 4 bytes/vector
        movec   VBR,a0                  ; get base address of vector table
        add.l   d1,a0                   ; add in the channel index
        moveq.l #0,d0                   ; clear a working register
        move.w  TPU_TICR+$FFF000,d0     ; get the interrupt configuration
        beq.s   tisu.0                  ; error - not programmed yet
        andi.w  #$FF,d0                 ; isolate the vector
        cmp.w   #$40,d0                 ; at an appropriate value?
        bmi.s   tisu.0
        lsl.l   #2,d0                   ; convert vector to byte offset
        add.l   d0,a0                   ; now have vector address in a0
        move.l  6(a7),(a0)              ; set up the vector
        moveq.l #0,d0                   ; set "no error" return
        rts
tisu.0  moveq.l #-1,d0                  ; set error return
        rts
